@@ -2,11 +2,11 @@
 r"""
 Interface with pyflatsurf
 """
-#*********************************************************************
+# ********************************************************************
 #  This file is part of sage-flatsurf.
 #
 #        Copyright (C) 2019      Vincent Delecroix
-#                      2019-2021 Julian Rüth
+#                      2019-2022 Julian Rüth
 #
 #  sage-flatsurf is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,9 +20,9 @@ Interface with pyflatsurf
 #
 #  You should have received a copy of the GNU General Public License
 #  along with sage-flatsurf. If not, see <https://www.gnu.org/licenses/>.
-#*********************************************************************
+# ********************************************************************
 
-from sage.all import QQ, AA
+from sage.all import QQ, AA, ZZ
 
 
 def _check_data(vp, fp, vec):
@@ -39,7 +39,7 @@ def _check_data(vp, fp, vec):
 
     n = len(vp) - 1
 
-    assert n%2 == 0, n
+    assert n % 2 == 0, n
     assert len(fp) == n+1
     assert len(vec) == n//2
 
@@ -61,9 +61,10 @@ def _check_data(vp, fp, vec):
         assert v.x() == 0, v.x()
         assert v.y() == 0, v.y()
 
+
 def _cycle_decomposition(p):
     n = len(p) - 1
-    assert n%2 == 0
+    assert n % 2 == 0
     cycles = []
     unseen = [True] * (n+1)
     for i in list(range(-n//2+1, 0)) + list(range(1, n//2)):
@@ -77,6 +78,7 @@ def _cycle_decomposition(p):
             cycles.append(cycle)
     return cycles
 
+
 def to_pyflatsurf(S):
     r"""
     Given S a translation surface from sage-flatsurf return a
@@ -117,7 +119,6 @@ def to_pyflatsurf(S):
         fp[e] = half_edge_labels[(t[0], j)]
         vp[fp[e]] = -e
 
-
     # convert the vp permutation into cycles
     verts = _cycle_decomposition(vp)
 
@@ -140,21 +141,60 @@ def to_pyflatsurf(S):
     from pyflatsurf.factory import make_surface
     return make_surface(verts, vec)
 
-def sage_base_ring(T):
+
+def sage_ring(surface):
     r"""
-    Given T a flatsurf::FlatTriangulation from libflatsurf/pyflatsurf, return a
-    SageMath base ring over which from_pyflatsurf() can construct a translation
-    surface.
+    Return the SageMath ring over which the pyflatsurf surface ``surface`` can
+    be constructed in sage-flatsurf.
 
     EXAMPLES::
 
         sage: from flatsurf import translation_surfaces
-        sage: from flatsurf.geometry.pyflatsurf_conversion import to_pyflatsurf, sage_base_ring # optional: pyflatsurf
+        sage: from flatsurf.geometry.pyflatsurf_conversion import to_pyflatsurf, sage_ring # optional: pyflatsurf
         sage: S = to_pyflatsurf(translation_surfaces.veech_double_n_gon(5)) # optional: pyflatsurf
-        sage: ring, to_ring = sage_base_ring(S) # optional: pyflatsurf
-        sage: ring # optional: pyflatsurf
+        sage: sage_ring(S) # optional: pyflatsurf
         Number Field in a with defining polynomial x^4 - 5*x^2 + 5 with a = 1.902113032590308?
 
+    """
+    from sage.all import Sequence
+    vectors = [surface.fromHalfEdge(e.positive()) for e in surface.edges()]
+    return Sequence([to_sage_ring(v.x()) for v in vectors] + [to_sage_ring(v.y()) for v in vectors]).universe()
+
+
+def to_sage_ring(x):
+    r"""
+    Given a coordinate of a flatsurf::Vector, return a SageMath element from
+    which :meth:`from_pyflatsurf` can eventually construct a translation surface.
+
+    EXAMPLES::
+
+        sage: from flatsurf.geometry.pyflatsurf_conversion import to_sage_ring  # optional: pyflatsurf
+        sage: to_sage_ring(1R).parent()  # optional: pyflatsurf
+        Integer Ring
+
+    GMP coordinate types::
+
+        sage: import cppyy  # optional: pyflatsurf
+        sage: import pyeantic  # optional: pyflatsurf
+        sage: to_sage_ring(cppyy.gbl.mpz_class(1)).parent()  # optional: pyflatsurf
+        Integer Ring
+        sage: to_sage_ring(cppyy.gbl.mpq_class(1, 2)).parent()  # optional: pyflatsurf
+        Rational Field
+
+    e-antic coordinate types::
+
+        sage: import pyeantic  # optional: pyflatsurf
+        sage: K = pyeantic.eantic.renf_class.make("a^3 - 3*a + 1", "a", "0.34 +/- 0.01", 64R)  # optional: pyflatsurf
+        sage: to_sage_ring(K.gen()).parent()  # optional: pyflatsurf
+        Number Field in a with defining polynomial x^3 - 3*x + 1 with a = 0.3472963553338607?
+
+    exact-real coordinate types::
+
+        sage: from pyexactreal import QQModule, RealNumber  # optional: pyflatsurf
+        sage: M = QQModule(RealNumber.random())   # optional: pyflatsurf
+        sage: to_sage_ring(M.gen(0R)).parent()  # optional: pyflatsurf
+        Real Numbers as (Rational Field)-Module
+
     """
     from flatsurf.features import cppyy_feature
     cppyy_feature.require()
@@ -167,29 +207,28 @@ def sage_base_ring(T):
             # The type constructed by t might not exist because the required C++ library has not been loaded.
             return None
 
-    coordinate = type(T.fromHalfEdge(1).x())
-    if coordinate is maybe_type(lambda: cppyy.gbl.mpz_class):
-        to_ring = ring = ZZ
-    elif coordinate is maybe_type(lambda: cppyy.gbl.mpq_class):
-        to_ring = ring = QQ
-    elif coordinate is maybe_type(lambda: cppyy.gbl.eantic.renf_elem_class):
+    if type(x) is int:
+        return ZZ(x)
+    elif type(x) is maybe_type(lambda: cppyy.gbl.mpz_class):
+        return ZZ(str(x))
+    elif type(x) is maybe_type(lambda: cppyy.gbl.mpq_class):
+        return QQ(str(x))
+    elif type(x) is maybe_type(lambda: cppyy.gbl.eantic.renf_elem_class):
         from pyeantic import RealEmbeddedNumberField
-        rng = RealEmbeddedNumberField(T.fromHalfEdge(1).x().parent())
-        ring = rng.number_field
-        to_ring = lambda x: ring(rng(x))
-    elif coordinate is maybe_type(lambda: cppyy.gbl.exactreal.Element[cppyy.gbl.exactreal.IntegerRing]):
+        real_embedded_number_field = RealEmbeddedNumberField(x.parent())
+        return real_embedded_number_field.number_field(real_embedded_number_field(x))
+    elif type(x) is maybe_type(lambda: cppyy.gbl.exactreal.Element[cppyy.gbl.exactreal.IntegerRing]):
         from pyexactreal import ExactReals
-        to_ring = ring = ExactReals(ZZ)
-    elif coordinate is maybe_type(lambda: cppyy.gbl.exactreal.Element[cppyy.gbl.exactreal.RationalField]):
+        return ExactReals(ZZ)(x)
+    elif type(x) is maybe_type(lambda: cppyy.gbl.exactreal.Element[cppyy.gbl.exactreal.RationalField]):
         from pyexactreal import ExactReals
-        to_ring = ring = ExactReals(QQ)
-    elif coordinate is maybe_type(lambda: cppyy.gbl.exactreal.Element[cppyy.gbl.exactreal.NumberField]):
+        return ExactReals(QQ)(x)
+    elif type(x) is maybe_type(lambda: cppyy.gbl.exactreal.Element[cppyy.gbl.exactreal.NumberField]):
         from pyexactreal import ExactReals
-        to_ring = ring = ExactReals(T.fromHalfEdge(1).x().module().ring().parameters)
+        return ExactReals(x.module().ring().parameters)(x)
     else:
-        raise NotImplementedError("unknown coordinate ring %s"%(coordinate,))
+        raise NotImplementedError(f"unknown coordinate ring for element {x} which is a {type(x)}")
 
-    return ring, to_ring
 
 def from_pyflatsurf(T):
     r"""
@@ -204,12 +243,33 @@ def from_pyflatsurf(T):
         sage: from_pyflatsurf(to_pyflatsurf(S)) # optional: pyflatsurf
         TranslationSurface built from 6 polygons
 
+    TESTS:
+
+    Verify that #137 has been resolved::
+
+        sage: from flatsurf import polygons
+        sage: from flatsurf.geometry.surface import Surface_list
+        sage: from flatsurf.geometry.translation_surface import TranslationSurface
+        sage: from flatsurf.geometry.gl2r_orbit_closure import GL2ROrbitClosure
+        sage: from flatsurf.geometry.pyflatsurf_conversion import from_pyflatsurf
+        sage: P = polygons.regular_ngon(10)
+        sage: S = Surface_list(P.base_ring())
+        sage: S.add_polygon(P)
+        0
+        sage: for i in range(5): S.set_edge_pairing(0, i, 0, 5+i)
+        sage: M = TranslationSurface(S)
+        sage: X = GL2ROrbitClosure(M)  # optional: pyflatsurf
+        sage: D0 = list(X.decompositions(2))[2]  # optional: pyflatsurf
+        sage: T0 = D0.triangulation()  # optional: pyflatsurf
+        sage: from_pyflatsurf(T0)  # optional: pyflatsurf
+        TranslationSurface built from 8 polygons
+
     """
     from flatsurf.features import pyflatsurf_feature
     pyflatsurf_feature.require()
     import pyflatsurf
 
-    ring, to_ring = sage_base_ring(T)
+    ring = sage_ring(T)
 
     from flatsurf.geometry.surface import Surface_list
     S = Surface_list(ring)
@@ -225,7 +285,7 @@ def from_pyflatsurf(T):
         a, b, c = map(pyflatsurf.flatsurf.HalfEdge, face)
 
         vectors = [T.fromHalfEdge(he) for he in face]
-        vectors = [V([to_ring(v.x()), to_ring(v.y())]) for v in vectors]
+        vectors = [V([ring(to_sage_ring(v.x())), ring(to_sage_ring(v.y()))]) for v in vectors]
         triangle = P(vectors)
         face_id = S.add_polygon(triangle)
 
